@@ -3,6 +3,10 @@ import sys
 from igo.util import FileMappedInputStream
 
 
+if sys.version_info[0] > 2:
+    unichr = chr
+
+
 class Node:
     """
     DoubleArrayのノード用の定数などが定義されているクラス
@@ -10,9 +14,6 @@ class Node:
     class Base:
         """ BASEノード用の定数およびメソッドが定義されているクラス """
 
-        INIT_VALUE = sys.maxint
-        """ BASEノードの初期値 """
-
         @staticmethod
         def ID(nid):
             """
@@ -26,16 +27,15 @@ class Node:
         CHECKノード用の定数が定義されているクラス
         """
         TERMINATE_CODE = 0
-
         """
-        文字列の終端を表す文字定数
+        文字列の終端を表す文字コード
 
         この文字はシステムにより予約されており、辞書内の形態素の表層形および解析対象テキストに含まれていた場合の動作は未定義
         """
 
         VACANT_CODE = 1
         """
-        CHECKノードが未使用だということを示すための文字定数
+        CHECKノードが未使用だということを示すための文字コード
 
         この文字はシステムにより予約されており、辞書内の形態素の表層形および解析対象テキストに含まれていた場合の動作は未定義
         """
@@ -58,21 +58,14 @@ class KeyStream:
         self.cur = start
         self.len = len(key)
 
-    @staticmethod
-    def compare(ks1, ks2):
-        return cmp(ks1.rest(), ks2.rest())
-
-    def startsWith(self, prefix, beg, length):
-        """
-        このメソッドは動作的には、{@code rest().startsWith(prefix.substring(beg, len))}、と等価。
-        ほんの若干だが、パフォーマンスを改善するために導入。
-        簡潔性のためになくしても良いかもしれない。
-        """
+    def startsWith(self, prefix):
+        cur = self.cur
         s = self.s
-        c = self.cur
-        if self.len - c < length:
+        length = len(prefix)
+
+        if self.len - cur < length:
             return False
-        return s[c:].startswith(prefix[beg:length])
+        return s[cur:cur + length] == prefix
 
     def rest(self):
         return self.s[self.cur:]
@@ -110,7 +103,7 @@ class Searcher:
             self.base = fmis.getIntArray(nodeSz)
             self.lens = fmis.getShortArray(tindSz)
             self.chck = fmis.getCharArray(nodeSz)
-            self.tail = fmis.getString(tailSz)
+            self.tail = fmis.getCharArray(tailSz)
         finally:
             fmis.close()
 
@@ -130,15 +123,18 @@ class Searcher:
         """
         base = self.base
         chck = self.chck
+        keyExists = self.keyExists
+
         node = base[0]
         kin = KeyStream(key)
+        code = kin.read()
         while 1:
             idx = node + code
             node = base[idx]
             if chck[idx] == code:
                 if node >= 0:
                     continue
-                elif kin.eos() or self.keyExists(kin, node):
+                elif kin.eos() or keyExists(kin, node):
                     return Node.Base.ID(node)
             return -1
 
@@ -161,12 +157,11 @@ class Searcher:
         while 1:
             code = kin.read()
             offset += 1
-            terminalIdx = node
+            terminalIdx = node + Node.Chck.TERMINATE_CODE
             if chck[terminalIdx] == Node.Chck.TERMINATE_CODE:
                 fn(start, offset, Node.Base.ID(base[terminalIdx]))
                 if code == Node.Chck.TERMINATE_CODE:
                     return
-            code = ord(code)
             idx = node + code
             node = base[idx]
             if chck[idx] == code:
@@ -178,11 +173,14 @@ class Searcher:
 
     def call_if_keyIncluding(self, kin, node, start, offset, fn):
         nodeId = Node.Base.ID(node)
-        if kin.startsWith(self.tail, self.begs[nodeId], self.lens[nodeId]):
-            fn(start, offset + self.lens[nodeId] + 1, nodeId)
+        l = self.lens[nodeId]
+        beg = self.begs[nodeId]
+        prefix = self.tail[beg:beg+l]
+        if kin.startsWith(prefix):
+            fn(start, offset + l + 1, nodeId)
 
     def keyExists(self, kin, node):
         nodeId = Node.Base.ID(node)
         beg = self.begs[nodeId]
-        s = self.tail.substring(beg, beg + self.lens[nodeId])
+        s = self.tail[beg:beg + self.lens[nodeId]]
         return kin.rest().equals(s)
